@@ -5,7 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -23,13 +23,13 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_socket_config.cpp,v 1.16 2004/06/20 07:35:51 suzhe Exp $
+ * $Id: scim_socket_config.cpp,v 1.22 2005/07/06 03:57:04 suzhe Exp $
  */
 
 #define Uses_SCIM_CONFIG_BASE
 #define Uses_SCIM_CONFIG_PATH
 #define Uses_SCIM_SOCKET
-#define Uses_SCIM_SOCKET_TRANSACTION
+#define Uses_SCIM_TRANSACTION
 #define Uses_C_STDIO
 #define Uses_C_STDLIB
 
@@ -42,6 +42,8 @@
 #define scim_config_module_init socket_LTX_scim_config_module_init
 #define scim_config_module_create_config socket_LTX_scim_config_module_create_config
 
+using namespace scim;
+
 extern "C" {
     void scim_module_init (void)
     {
@@ -58,16 +60,17 @@ extern "C" {
         SCIM_DEBUG_CONFIG(1) << "Initializing Socket Config module (more)...\n";
     }
 
-    ConfigPointer scim_config_module_create_config (const String &app_name)
+    ConfigPointer scim_config_module_create_config ()
     {
         SCIM_DEBUG_CONFIG(1) << "Creating a Socket Config instance...\n";
-        return new SocketConfig (app_name);
+        return new SocketConfig ();
     }
 }
 
-SocketConfig::SocketConfig (const String& app_name)
-    : ConfigBase (app_name),
-      m_valid (false),
+namespace scim {
+
+SocketConfig::SocketConfig ()
+    : m_valid (false),
       m_socket_address (scim_get_default_socket_config_address ()),
       m_socket_timeout (scim_get_default_socket_timeout ()),
       m_connected (false)
@@ -87,6 +90,12 @@ SocketConfig::valid () const
     return ConfigBase::valid() && m_valid;
 }
 
+String
+SocketConfig::get_name () const
+{
+    return "socket";
+}
+
 // String
 bool
 SocketConfig::read (const String& key, String *pStr) const
@@ -94,7 +103,8 @@ SocketConfig::read (const String& key, String *pStr) const
     if (!valid () || !pStr || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
+
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -127,7 +137,7 @@ SocketConfig::read (const String& key, int *pl) const
     if (!valid () || !pl || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -163,7 +173,7 @@ SocketConfig::read (const String& key, double* val) const
     if (!valid () || !val || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -199,7 +209,7 @@ SocketConfig::read (const String& key, bool* val) const
     if (!valid () || !val || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
     
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -237,7 +247,7 @@ SocketConfig::read (const String& key, std::vector <String>* val) const
     
     val->clear ();
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -272,7 +282,7 @@ SocketConfig::read (const String& key, std::vector <int>* val) const
     
     val->clear();
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -308,7 +318,7 @@ SocketConfig::write (const String& key, const String& value)
     if (!valid () || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -339,7 +349,7 @@ SocketConfig::write (const String& key, int value)
     if (!valid () || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -373,7 +383,7 @@ SocketConfig::write (const String& key, double value)
     char buf [256];
     snprintf (buf, 255, "%lE", value);
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -404,7 +414,7 @@ SocketConfig::write (const String& key, bool value)
     if (!valid () || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -435,7 +445,7 @@ SocketConfig::write (const String& key, const std::vector <String>& value)
     if (!valid () || key.empty()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -471,7 +481,7 @@ SocketConfig::write (const String& key, const std::vector <int>& value)
     for (uint32 i=0; i<value.size (); ++i)
         vec.push_back ((uint32) value [i]);
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -503,7 +513,7 @@ SocketConfig::flush()
     if (!valid ()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -534,7 +544,7 @@ SocketConfig::erase (const String& key)
     if (!valid ()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -564,7 +574,7 @@ SocketConfig::reload ()
     if (!valid ()) return false;
     if (!m_connected && !open_connection ()) return false;
 
-    SocketTransaction trans;
+    Transaction trans;
     int cmd;
 
     for (int retry = 0; retry < 3; ++retry) {
@@ -604,7 +614,7 @@ SocketConfig::reload ()
 }
 
 void
-SocketConfig::init_transaction (SocketTransaction &trans) const
+SocketConfig::init_transaction (Transaction &trans) const
 {
     trans.clear ();
     trans.put_command (SCIM_TRANS_CMD_REQUEST);
@@ -625,11 +635,11 @@ SocketConfig::open_connection () const
     }
 
     // Init the connection,
-    if (!scim_socket_trans_open_connection (m_socket_magic_key,
-                                            String ("SocketConfig"),
-                                            String ("SocketFrontEnd"),
-                                            m_socket_client,
-                                            m_socket_timeout)) {
+    if (!scim_socket_open_connection (m_socket_magic_key,
+                                      String ("SocketConfig"),
+                                      String ("SocketFrontEnd"),
+                                      m_socket_client,
+                                      m_socket_timeout)) {
         m_socket_client.close ();
         return false;
     }
@@ -639,6 +649,8 @@ SocketConfig::open_connection () const
     return true;
 }
 
+} // namespace scim
+
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
