@@ -5,8 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2004 James Su <suzhe@tsinghua.org.cn>
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -24,7 +23,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_panel_gtk_setup.cpp,v 1.11 2004/06/26 06:32:55 suzhe Exp $
+ * $Id: scim_panel_gtk_setup.cpp,v 1.14 2005/01/10 08:30:48 suzhe Exp $
  *
  */
 
@@ -57,7 +56,7 @@ using namespace scim;
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_FACTORY_ICON "/Panel/Gtk/ToolBar/ShowFactoryIcon"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_FACTORY_NAME "/Panel/Gtk/ToolBar/ShowFactoryName"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_STICK_ICON   "/Panel/Gtk/ToolBar/ShowStickIcon"
-#define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_SETUP_ICON   "/Panel/Gtk/ToolBar/ShowSetupIcon"
+#define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_MENU_ICON    "/Panel/Gtk/ToolBar/ShowMenuIcon"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_HELP_ICON    "/Panel/Gtk/ToolBar/ShowHelpIcon"
 #define SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_PROPERTY_LABEL "/Panel/Gtk/ToolBar/ShowPropertyLabel"
 #define SCIM_CONFIG_PANEL_GTK_LOOKUP_TABLE_EMBEDDED     "/Panel/Gtk/LookupTableEmbedded"
@@ -120,11 +119,11 @@ extern "C" {
 static bool   __config_toolbar_always_show       = false;
 static bool   __config_toolbar_auto_snap         = false;
 static int    __config_toolbar_hide_timeout      = 2;
-static bool   __config_toolbar_show_factory_icon  = true;
-static bool   __config_toolbar_show_factory_name  = true;
-static bool   __config_toolbar_show_stick_icon   = true;
-static bool   __config_toolbar_show_setup_icon   = true;
-static bool   __config_toolbar_show_help_icon    = true;
+static bool   __config_toolbar_show_factory_icon = true;
+static bool   __config_toolbar_show_factory_name = true;
+static bool   __config_toolbar_show_stick_icon   = false;
+static bool   __config_toolbar_show_menu_icon    = true;
+static bool   __config_toolbar_show_help_icon    = false;
 static bool   __config_toolbar_show_property_label = true;
 static bool   __config_lookup_table_embedded     = true;
 static bool   __config_lookup_table_vertical     = false;
@@ -141,7 +140,7 @@ static GtkWidget * __widget_toolbar_hide_timeout      = 0;
 static GtkWidget * __widget_toolbar_show_factory_icon  = 0;
 static GtkWidget * __widget_toolbar_show_factory_name  = 0;
 static GtkWidget * __widget_toolbar_show_stick_icon   = 0;
-static GtkWidget * __widget_toolbar_show_setup_icon   = 0;
+static GtkWidget * __widget_toolbar_show_menu_icon   = 0;
 static GtkWidget * __widget_toolbar_show_help_icon    = 0;
 static GtkWidget * __widget_toolbar_show_property_label = 0;
 static GtkWidget * __widget_lookup_table_embedded     = 0;
@@ -255,9 +254,9 @@ create_setup_window ()
                           (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                           (GtkAttachOptions) (GTK_EXPAND), 4, 0);
 
-        __widget_toolbar_show_setup_icon = gtk_check_button_new_with_mnemonic (_("Show s_etup icon"));
-        gtk_widget_show (__widget_toolbar_show_setup_icon);
-        gtk_table_attach (GTK_TABLE (table), __widget_toolbar_show_setup_icon, 1, 2, 2, 3,
+        __widget_toolbar_show_menu_icon = gtk_check_button_new_with_mnemonic (_("Show m_enu icon"));
+        gtk_widget_show (__widget_toolbar_show_menu_icon);
+        gtk_table_attach (GTK_TABLE (table), __widget_toolbar_show_menu_icon, 1, 2, 2, 3,
                           (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                           (GtkAttachOptions) (GTK_EXPAND), 4, 0);
 
@@ -352,9 +351,9 @@ create_setup_window ()
                           G_CALLBACK (on_default_toggle_button_toggled),
                           &__config_toolbar_show_stick_icon);
 
-        g_signal_connect ((gpointer) __widget_toolbar_show_setup_icon, "toggled",
+        g_signal_connect ((gpointer) __widget_toolbar_show_menu_icon, "toggled",
                           G_CALLBACK (on_default_toggle_button_toggled),
-                          &__config_toolbar_show_setup_icon);
+                          &__config_toolbar_show_menu_icon);
 
         g_signal_connect ((gpointer) __widget_toolbar_show_help_icon, "toggled",
                           G_CALLBACK (on_default_toggle_button_toggled),
@@ -415,9 +414,9 @@ create_setup_window ()
                               _("If this option is checked, "
                                 "the stick icon will be showed on the toolbar."), NULL);
 
-        gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_show_setup_icon,
+        gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_show_menu_icon,
                               _("If this option is checked, "
-                                "the setup icon will be showed on the toolbar."), NULL);
+                                "the menu icon will be showed on the toolbar."), NULL);
 
         gtk_tooltips_set_tip (__widget_tooltips, __widget_toolbar_show_help_icon,
                               _("If this option is checked, "
@@ -501,10 +500,10 @@ setup_widget_value ()
             __config_toolbar_show_stick_icon);
     }
 
-    if (__widget_toolbar_show_setup_icon) {
+    if (__widget_toolbar_show_menu_icon) {
         gtk_toggle_button_set_active (
-            GTK_TOGGLE_BUTTON (__widget_toolbar_show_setup_icon),
-            __config_toolbar_show_setup_icon);
+            GTK_TOGGLE_BUTTON (__widget_toolbar_show_menu_icon),
+            __config_toolbar_show_menu_icon);
     }
 
     if (__widget_toolbar_show_help_icon) {
@@ -572,9 +571,9 @@ load_config (const ConfigPointer &config)
         __config_toolbar_show_stick_icon =
             config->read (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_STICK_ICON),
                           __config_toolbar_show_stick_icon);
-        __config_toolbar_show_setup_icon =
-            config->read (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_SETUP_ICON),
-                          __config_toolbar_show_setup_icon);
+        __config_toolbar_show_menu_icon =
+            config->read (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_MENU_ICON),
+                          __config_toolbar_show_menu_icon);
         __config_toolbar_show_help_icon =
             config->read (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_HELP_ICON),
                           __config_toolbar_show_help_icon);
@@ -619,8 +618,8 @@ save_config (const ConfigPointer &config)
                        __config_toolbar_show_factory_name);
         config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_STICK_ICON),
                        __config_toolbar_show_stick_icon);
-        config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_SETUP_ICON),
-                       __config_toolbar_show_setup_icon);
+        config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_MENU_ICON),
+                       __config_toolbar_show_menu_icon);
         config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_HELP_ICON),
                        __config_toolbar_show_help_icon);
         config->write (String (SCIM_CONFIG_PANEL_GTK_TOOLBAR_SHOW_PROPERTY_LABEL),
