@@ -1,7 +1,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -19,13 +19,14 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_debug.cpp,v 1.5 2004/06/09 15:13:40 suzhe Exp $
+ * $Id: scim_debug.cpp,v 1.9 2005/07/08 05:06:43 suzhe Exp $
  *
  */
 
 #define Uses_SCIM_DEBUG
 #include "scim_private.h"
 #include "scim.h"
+#include <cstdio>
 
 namespace scim {
 
@@ -57,6 +58,17 @@ std::ostream * DebugOutput::output_stream = &std::cerr;
 
 static std::ofstream __debug_output_file;
 
+#if ENABLE_DEBUG
+DebugOutput::DebugOutput (uint32 mask, uint32 verbose)
+    : m_mask (mask), m_verbose (verbose)
+{
+}
+#else
+DebugOutput::DebugOutput (uint32, uint32)
+{
+}
+#endif
+
 void
 DebugOutput::set_verbose_level (uint32 verbose)
 {
@@ -122,6 +134,15 @@ DebugOutput::set_output (const String &file)
     }
 }
 
+String
+DebugOutput::serial_number ()
+{
+    static unsigned int serial = 0;
+    char buf [40];
+    snprintf (buf, 40, "<%08u>:", serial ++);
+    return String (buf);
+}
+
 } // namespace scim
 
 /*
