@@ -9,9 +9,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2004 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2003 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -29,7 +27,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_debug.h,v 1.13 2004/06/09 15:13:40 suzhe Exp $
+ * $Id: scim_debug.h,v 1.17 2005/07/08 05:06:43 suzhe Exp $
  */
 
 #ifndef __SCIM_DEBUG_H
@@ -66,7 +64,7 @@ namespace scim {
  *
  * @{
  */
-#define SCIM_DEBUG(mask,level)        (scim::DebugOutput(mask,level) << __FILE__ << ":" << __LINE__ << " > ")
+#define SCIM_DEBUG(mask,level)        (scim::DebugOutput(mask,level) << scim::DebugOutput::serial_number () << __FILE__ << ":" << __LINE__ << " > ")
 #define SCIM_DEBUG_MAIN(level)        SCIM_DEBUG(SCIM_DEBUG_MainMask,level)
 #define SCIM_DEBUG_CONFIG(level)      SCIM_DEBUG(SCIM_DEBUG_ConfigMask,level)
 #define SCIM_DEBUG_IMENGINE(level)    SCIM_DEBUG(SCIM_DEBUG_IMEngineMask,level)
@@ -101,8 +99,7 @@ public:
      * @param mask - the debug filter mask.
      * @param verbose - the verbose level of the debug message.
      */
-    DebugOutput (uint32 mask = SCIM_DEBUG_AllMask, uint32 verbose = 1)
-        : m_mask (mask), m_verbose (verbose) { }
+    DebugOutput (uint32 mask = SCIM_DEBUG_AllMask, uint32 verbose = 1);
 
     /**
      * @brief A template stream output operator.
@@ -162,6 +159,8 @@ public:
      *               output will be set to std::cout.
      */
     static void set_output (const String &file);
+
+    static String serial_number ();
 };
 #else
 class DebugOutput
@@ -172,10 +171,10 @@ private:
     static std::ostream   *output_stream;
 
 public:
-    DebugOutput (uint32 mask = SCIM_DEBUG_AllMask, uint32 verbose = 1){ }
+    DebugOutput (uint32 mask = SCIM_DEBUG_AllMask, uint32 verbose = 1);
 
     template <typename T>
-    const DebugOutput& operator << (const T& t) const {
+    const DebugOutput& operator << (const T&) const {
         return *this;
     }
 
@@ -188,6 +187,8 @@ public:
 
     static void set_verbose_level (uint32 verbose);
     static void set_output (const String &file);
+
+    static String serial_number ();
 };
 #endif
 
