@@ -6,10 +6,7 @@
 /* 
  * Smart Common Input Method
  * 
- * Copyright (c) 2004 James Su <suzhe@tsinghua.org.cn>
- * Copyright (c) 2004 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2003 James Su <suzhe@turbolinux.com.cn>
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -27,7 +24,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_frontend.h,v 1.29 2004/07/23 16:17:30 suzhe Exp $
+ * $Id: scim_frontend.h,v 1.41 2005/06/26 16:35:33 suzhe Exp $
  */
 
 #ifndef __SCIM_FRONTEND_H
@@ -107,7 +104,61 @@ protected:
      *
      * @return the number of IMEngine factories found.
      */
-    uint32 get_factory_list (std::vector<String> &uuids, const String &encoding) const;
+    uint32 get_factory_list_for_encoding (std::vector<String> &uuids, const String &encoding) const;
+
+    /**
+     * @brief Get the IMEngine factories list for specific language
+     *
+     * @param uuids    the vector to store the factories' uuids which
+     *                 support the encoding.
+     * @param language the language to be queried. If empty,
+     *                 all IMEngine factories will be returned.
+     *
+     * @return the number of IMEngine factories found.
+     */
+    uint32 get_factory_list_for_language (std::vector<String> &uuids, const String &language) const;
+
+    /**
+     * @brief Get the default IMEngineFactory UUID for a specific language and encoding.
+     *
+     * @param language the language to be queried.
+     * @param encoding the encoding to be queried, if empty then don't match encoding.
+     *
+     * @return the UUID of the default IMEngineFactory for this language.
+     */
+    String get_default_factory (const String &language, const String &encoding) const;
+
+    /**
+     * @brief Set the default IMEngineFactory for a specific language.
+     *
+     * @param language the language to be set.
+     * @param the uuid of the default IMEngineFactory for this language.
+     */
+    void set_default_factory (const String &language, const String &uuid);
+
+    /**
+     * @brief Get the next IMEngineFactory for a specific language and encoding.
+     *
+     * @param language the language to be queried, if empty then don't match language.
+     * @param encoding the encoding to be queried, if empty then don't match encoding.
+     * @param cur_uuid the UUID of current IMEngineFactory.
+     *
+     * @return the UUID of the next IMEngineFactory for this language and encoding
+     *         corresponding to the current IMEngineFactory.
+     */
+    String get_next_factory (const String &language, const String &encoding, const String &cur_uuid) const;
+
+    /**
+     * @brief Get the previous IMEngineFactory for a specific language and encoding.
+     *
+     * @param language the language to be queried, if empty then don't match language.
+     * @param encoding the encoding to be queried, if empty then don't match encoding.
+     * @param cur_uuid the UUID of current IMEngineFactory.
+     *
+     * @return the UUID of the previous IMEngineFactory for this language and encoding
+     *         corresponding to the current IMEngineFactory.
+     */
+    String get_previous_factory (const String &language, const String &encoding, const String &cur_uuid) const;
 
     /**
      * @brief get the name of an IMEngine factory.
@@ -159,6 +210,15 @@ protected:
      */
     String get_factory_language (const String &uuid) const;
 
+    /**
+     * @brief Check if an IMEngine factory is valid and the given encoding is supported by it.
+     *
+     * @param uuid The uuid of the IMEngine factory to be checked.
+     * @param encoding The encoding should be supported by the factory.
+     * @return true if the factory is valid and the given encoding is supported.
+     */
+    bool validate_factory (const String &uuid, const String &encoding = String ("")) const;
+
     /**
      * @brief get all locales supported by BackEnd.
      * @return a comman separated list of all supported locales.
@@ -184,6 +244,7 @@ protected:
      *
      * @param si_id the IMEngine instance to be replaced.
      * @param sf_uuid the new IMEngine factory to be used.
+     * @return true if success.
      */
     bool replace_instance (int si_id, const String &sf_uuid);
 
@@ -201,6 +262,13 @@ protected:
      */
     void delete_all_instances ();
 
+    /**
+     * @brief get the factory uuid of this instance.
+     * @param id the IMEngine instance id.
+     * @return the factory uuid of this instance.
+     */
+    String get_instance_uuid (int id) const;
+
     /**
      * @brief get the working encoding of an IMEngine instance.
      * @param id the IMEngine instance id.
@@ -313,13 +381,28 @@ protected:
      */
     void trigger_property (int id, const String &property) const;
 
+    /**
+     * @brief let a specific IMEngine instance to process the events sent from a Helper process.
+     * @param id the id of the IMEngine instance.
+     * @param helper_uuid the uuid of the Helper process.
+     * @param trans the transaction which contains the events.
+     */
+    void process_helper_event (int id, const String &helper_uuid, const Transaction &trans) const;
+
+    /**
+     * @brief let a specific IMEngine instance to update itself according to the capabilities of the client application.
+     * @param id the id of the IMEngine instance.
+     * @param cap the bitset of the capabilities which are supported by client.
+     */
+    void update_client_capabilities (int id, unsigned int cap) const;
+
     /**
      * @}
      */
 
 protected:
     /**
-     * @name Pure virtual protected methods.
+     * @name Virtual protected methods.
      *
      * The following methods should be implemented by derivation classes.
      * these functions handle the real things.
@@ -331,44 +414,44 @@ protected:
      * @brief show preedit string area for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      */
-    virtual void show_preedit_string (int id) = 0;
+    virtual void show_preedit_string (int id);
 
     /**
      * @brief show aux string area for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      */
-    virtual void show_aux_string     (int id) = 0;
+    virtual void show_aux_string     (int id);
 
     /**
      * @brief show lookup table area for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      */
-    virtual void show_lookup_table   (int id) = 0;
+    virtual void show_lookup_table   (int id);
 
     /**
      * @brief hide preedit string area for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      */
-    virtual void hide_preedit_string (int id) = 0;
+    virtual void hide_preedit_string (int id);
 
     /**
      * @brief hide aux string area for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      */
-    virtual void hide_aux_string     (int id) = 0;
+    virtual void hide_aux_string     (int id);
 
     /**
      * @brief hide lookup table area for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      */
-    virtual void hide_lookup_table   (int id) = 0;
+    virtual void hide_lookup_table   (int id);
 
     /**
      * @brief update the position of preedit caret for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      * @param caret the new caret position.
      */
-    virtual void update_preedit_caret  (int id, int caret) = 0;
+    virtual void update_preedit_caret  (int id, int caret);
 
     /**
      * @brief update the content of preedit string for an IMEngine instance.
@@ -376,7 +459,7 @@ protected:
      * @param str the new content of preedit string.
      * @param attrs the string attributes.
      */
-    virtual void update_preedit_string (int id, const WideString & str, const AttributeList & attrs) = 0;
+    virtual void update_preedit_string (int id, const WideString & str, const AttributeList & attrs);
 
     /**
      * @brief update the content of aux string for an IMEngine instance.
@@ -384,43 +467,115 @@ protected:
      * @param str the new content of aux string.
      * @param attrs the string attributes.
      */
-    virtual void update_aux_string     (int id, const WideString & str, const AttributeList & attrs) = 0;
+    virtual void update_aux_string     (int id, const WideString & str, const AttributeList & attrs);
 
     /**
      * @brief update the content of lookup table for an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      * @param table the new lookup table.
      */
-    virtual void update_lookup_table   (int id, const LookupTable & table) = 0;
+    virtual void update_lookup_table   (int id, const LookupTable & table);
 
     /**
      * @brief commit a string to client for an IMEngine instance.
      * @param id the id of the IMEngine instance to commit the string.
      * @param str the string to be committed.
      */
-    virtual void commit_string         (int id, const WideString & str) = 0;
+    virtual void commit_string         (int id, const WideString & str);
 
     /**
      * @brief forward a keyevent to the client of an IMEngine instance.
      * @param id the id of the IMEngine instance.
      * @param key the key event to be forwarded.
      */
-    virtual void forward_key_event      (int id, const KeyEvent & key) = 0;
+    virtual void forward_key_event      (int id, const KeyEvent & key);
 
     /**
      * @brief register all the properties of an IMEngine instance into this FrontEnd.
      * @param id the id of the IMEngine instance. It must have been focused in.
      * @param properties the PropertyList contains all the properties of this IMEngine instance.
      */
-    virtual void register_properties   (int id, const PropertyList & properties) = 0;
+    virtual void register_properties   (int id, const PropertyList & properties);
 
     /**
      * @brief update a property of an IMEngine instance.
      * @param id the id of the IMEngine instance. It must have been focused in.
      * @param property the Property to be updated.
      */
-    virtual void update_property       (int id, const Property & property) = 0;
+    virtual void update_property       (int id, const Property & property);
 
+    /**
+     * @brief generate a short beep.
+     * @param id the id of the IMEngine instance. It must have been focused in.
+     */
+    virtual void beep                  (int id);
+
+    /**
+     * @brief start a Client Helper process.
+     *
+     * @param id the id of the IMEngine instance. It must have been focused in.
+     * @param helper_uuid The UUID of the Helper object.
+     */
+    virtual void start_helper          (int id, const String &helper_uuid);
+
+    /**
+     * @brief stop a Client Helper process.
+     *
+     * @param id the id of the IMEngine instance. It must have been focused in.
+     * @param helper_uuid The UUID of the Helper object.
+     */
+    virtual void stop_helper           (int id, const String &helper_uuid);
+
+    /**
+     * @brief send an events transaction to a Client Helper process.
+     *
+     * @param id the id of the IMEngine instance. It must have been focused in.
+     * @param helper_uuid The UUID of the Helper object.
+     * @param trans the transaction which contains events.
+     */
+    virtual void send_helper_event     (int id, const String &helper_uuid, const Transaction &trans);
+
+    /**
+     * @brief Retrieves context around the insertion point.
+     * 
+     * Input methods typically want context in order to constrain
+     * input text based on existing text;
+     * this is important for languages such as Thai where
+     * only some sequences of characters are allowed.
+     *
+     * Unlike other signal activation actions, this action will return the result
+     * immediately.
+     *
+     * @param id            the id of the IMEngine instance. It must have been focused in.
+     * @param text          location to store the context string around the insertion point.
+     * @param cursor        location to store index of the insertion cursor within @text.
+     * @param maxlen_before the maxmium length of context string to be retrieved
+     *                      before the cursor; -1 means unlimited.
+     * @param maxlen_after  the maxmium length of context string to be retrieved
+     *                      after the cursor; -1 means unlimited.
+     *
+     * @return true if surrounding text was provided.
+     */
+    virtual bool get_surrounding_text  (int id, WideString &text, int &cursor, int maxlen_before, int maxlen_after);
+
+    /**
+     * @brief Ask the client to delete characters around the cursor position.
+     * 
+     * In order to use this function, you should first call
+     * get_surrounding_text () to get the current context, and
+     * call this function immediately afterwards to make sure that you
+     * know what you are deleting. You should also account for the fact
+     * that even if the signal was handled, the input context might not
+     * have deleted all the characters that were requested to be deleted.
+     *
+     * @param id     the id of the IMEngine instance. It must have been focused in.
+     * @param offset offset from cursor position in chars;
+     *               a negative value means start before the cursor.
+     * @param len    number of characters to delete.
+     *
+     * @return true if the signal was handled.
+     */
+    virtual bool delete_surrounding_text  (int id, int offset, int len);
     /**
      * @}
      */
