@@ -5,7 +5,7 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002 James Su <suzhe@turbolinux.com.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
  *
  * This library is free software; you can redistribute it and/or
@@ -23,11 +23,12 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_socket.cpp,v 1.31.2.1 2004/11/08 01:57:49 suzhe Exp $
+ * $Id: scim_socket.cpp,v 1.43 2005/01/25 15:13:15 suzhe Exp $
  *
  */
 
 #define Uses_SCIM_SOCKET
+#define Uses_SCIM_TRANSACTION
 #define Uses_SCIM_CONFIG_PATH
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -584,7 +585,9 @@ public:
             // Unlink the socket file.
             if (m_binded && m_family == SCIM_SOCKET_LOCAL) {
                 const struct sockaddr_un * data = static_cast <const struct sockaddr_un *>(m_address.get_data ());
-                SCIM_DEBUG_SOCKET(3) << "  Unlinking socket file " << data->sun_path << "...\n";
+                // FIXME: Don't unlink the socket file, because if the process is forked and child process exits
+                // the socket file will be unlinked by child process.
+                //SCIM_DEBUG_SOCKET(3) << "  Unlinking socket file " << data->sun_path << "...\n";
                 ::unlink (data->sun_path);
             }
         }
@@ -704,6 +707,8 @@ struct SocketServer::SocketServerImpl
     int      num_clients;
     int      max_clients;
 
+    std::vector <int> ext_fds;
+
     SocketServerSignalSocket accept_signal;
     SocketServerSignalSocket receive_signal;
     SocketServerSignalSocket exception_signal;
@@ -732,7 +737,7 @@ SocketServer::~SocketServer ()
 }
 
 bool
-SocketServer::valid ()
+SocketServer::valid () const
 {
     return m_impl->created;
 }
@@ -889,6 +894,9 @@ SocketServer::shutdown ()
 
         m_impl->running = false;
 
+        for (int i = 0; i<m_impl->ext_fds.size (); i++)
+            FD_CLR (m_impl->ext_fds [i], &m_impl->active_fds);
+
         for (int i = 0; i<m_impl->max_fd + 1; i++) {
             //Close all client
             if (FD_ISSET (i, &(m_impl->active_fds)) && i != Socket::get_id ()) {
@@ -901,27 +909,32 @@ SocketServer::shutdown ()
         m_impl->created = false;
         m_impl->err = 0;
         m_impl->num_clients = 0;
+        m_impl->ext_fds.clear ();
         FD_ZERO (&(m_impl->active_fds));
 
         Socket::close ();
     }
 }
 
-void
+bool
 SocketServer::close_connection (const Socket &socket)
 {
     int id = socket.get_id ();
     if (m_impl->created && m_impl->running && id > 0 && FD_ISSET (id, &(m_impl->active_fds))) {
 
-        SCIM_DEBUG_SOCKET (2) << " SocketServer: Closing the connection: "
-                              << socket.get_id () << "\n";
+        SCIM_DEBUG_SOCKET (2) << " SocketServer: Closing the connection: " << id << "\n";
 
         m_impl->num_clients --;
 
         FD_CLR (id, &(m_impl->active_fds));
 
+        std::vector <int>::iterator it = std::find (m_impl->ext_fds.begin (), m_impl->ext_fds.end (), id);
+        if (it != m_impl->ext_fds.end ()) m_impl->ext_fds.erase (it);
+
         ::close (id);
+        return true;
     }
+    return false;
 }
 
 int
@@ -955,6 +968,38 @@ SocketServer::set_max_clients (int max_clients)
         m_impl->max_clients = max_clients;
 }
 
+bool
+SocketServer::insert_external_socket (const Socket &sock)
+{
+    int fd = sock.get_id ();
+
+    if (valid () && sock.valid () && sock.wait_for_data (0) >= 0 &&
+        m_impl->num_clients < m_impl->max_clients &&
+        !FD_ISSET (fd, &m_impl->active_fds)) {
+        m_impl->ext_fds.push_back (fd);
+        FD_SET (fd, &m_impl->active_fds);
+        if (m_impl->max_fd < fd) m_impl->max_fd = fd;
+        m_impl->num_clients ++;
+        return true;
+    }
+    return false;
+}
+
+bool
+SocketServer::remove_external_socket (const Socket &sock)
+{
+    int fd = sock.get_id ();
+
+    if (valid () && FD_ISSET (fd, &m_impl->active_fds)) {
+        FD_CLR (fd, &m_impl->active_fds);
+        std::vector <int>::iterator it = std::find (m_impl->ext_fds.begin (), m_impl->ext_fds.end (), fd);
+        if (it != m_impl->ext_fds.end ()) m_impl->ext_fds.erase (it);
+        m_impl->num_clients --;
+        return true;
+    }
+    return false;
+}
+
 Connection
 SocketServer::signal_connect_accept (SocketServerSlotSocket *slot)
 {
@@ -1017,8 +1062,9 @@ SocketClient::close ()
     m_connected = false;
 }
 
-#define SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS    "local:/tmp/scim-socket-frontend"
-#define SCIM_DEFAULT_PANEL_SOCKET_ADDRESS       "local:/tmp/scim-panel-socket"
+#define SCIM_DEFAULT_SOCKET_FRONTEND_ADDRESS        "local:/tmp/scim-socket-frontend"
+#define SCIM_DEFAULT_PANEL_SOCKET_ADDRESS           "local:/tmp/scim-panel-socket"
+#define SCIM_DEFAULT_HELPER_MANAGER_SOCKET_ADDRESS  "local:/tmp/scim-helper-manager-socket"
 
 String scim_get_default_socket_frontend_address ()
 {
@@ -1083,7 +1129,7 @@ String scim_get_default_socket_config_address ()
     return address;
 }
 
-String scim_get_default_panel_socket_address ()
+String scim_get_default_panel_socket_address (const String &display)
 {
     String address (SCIM_DEFAULT_PANEL_SOCKET_ADDRESS);
 
@@ -1098,6 +1144,36 @@ String scim_get_default_panel_socket_address ()
     if (address == "default")
         address = SCIM_DEFAULT_PANEL_SOCKET_ADDRESS;
 
+    String::size_type colon_pos = display.rfind (':');
+
+    String disp_name = display;
+
+    // Maybe It's a X11 DISPLAY name
+    if (colon_pos != String::npos) {
+        String::size_type dot_pos = display.find ('.', colon_pos);
+        // It has screen number
+        if (dot_pos != String::npos) {
+            disp_name = display.substr (0, dot_pos);
+        }
+    }
+
+    return address + String ("-") + disp_name;
+}
+
+String scim_get_default_helper_manager_socket_address ()
+{
+    String address (SCIM_DEFAULT_HELPER_MANAGER_SOCKET_ADDRESS);
+
+    address = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_HELPER_MANAGER_SOCKET_ADDRESS, address);
+
+    const char *env = getenv ("SCIM_HELPER_MANAGER_SOCKET_ADDRESS");
+    if (env && strlen (env) > 0) {
+        address = String (env);
+    }
+
+    if (address == "default")
+        address = SCIM_DEFAULT_HELPER_MANAGER_SOCKET_ADDRESS;
+
     return address;
 }
 
@@ -1117,6 +1193,96 @@ int scim_get_default_socket_timeout ()
     return timeout;
 }
 
+static bool
+scim_socket_check_type (const String &types,
+                        const String &atype)
+{
+    std::vector <String> type_list;
+    scim_split_string_list (type_list, types, ',');
+
+    return std::find (type_list.begin (), type_list.end (), atype) != type_list.end ();
+}
+
+bool
+scim_socket_open_connection   (uint32       &key,
+                               const String &client_type,
+                               const String &server_type,
+                               const Socket &socket,
+                               int           timeout)
+{
+    if (!socket.valid () || !client_type.length () || !server_type.length ())
+        return false;
+
+    Transaction trans;
+
+    trans.put_command (SCIM_TRANS_CMD_REQUEST);
+    trans.put_command (SCIM_TRANS_CMD_OPEN_CONNECTION);
+    trans.put_data (String (SCIM_BINARY_VERSION));
+    trans.put_data (client_type);
+
+    if (trans.write_to_socket (socket)) {
+        int cmd;
+        String server_types;
+        if (trans.read_from_socket (socket, timeout) &&
+            trans.get_command (cmd) && cmd == SCIM_TRANS_CMD_REPLY &&
+            trans.get_data (server_types) && scim_socket_check_type (server_types, server_type) &&
+            trans.get_data (key)) {
+            trans.clear ();
+            trans.put_command (SCIM_TRANS_CMD_REPLY);
+            trans.put_command (SCIM_TRANS_CMD_OK);
+            if (trans.write_to_socket (socket))
+                return true;
+        } else {
+            trans.clear ();
+            trans.put_command (SCIM_TRANS_CMD_REPLY);
+            trans.put_command (SCIM_TRANS_CMD_FAIL);
+            trans.write_to_socket (socket);
+        }
+    }
+
+    return false;
+}
+
+String
+scim_socket_accept_connection (uint32       &key,
+                               const String &server_types,
+                               const String &client_types,
+                               const Socket &socket,
+                               int           timeout)
+{
+    if (!socket.valid () || !client_types.length () || !server_types.length ())
+        return String ("");
+
+    Transaction trans;
+
+    if (trans.read_from_socket (socket, timeout)) {
+        int cmd;
+        String version;
+        String client_type;
+        if (trans.get_command (cmd)  && cmd == SCIM_TRANS_CMD_REQUEST &&
+            trans.get_command (cmd)  && cmd == SCIM_TRANS_CMD_OPEN_CONNECTION &&
+            trans.get_data (version) && version == String (SCIM_BINARY_VERSION) &&
+            trans.get_data (client_type) && 
+            (scim_socket_check_type (client_types, client_type) || client_type == "ConnectionTester")) {
+            key = (uint32) rand ();
+            trans.clear ();
+            trans.put_command (SCIM_TRANS_CMD_REPLY);
+            trans.put_data (server_types);
+            trans.put_data (key);
+
+            if (trans.write_to_socket (socket) &&
+                trans.read_from_socket (socket, timeout) &&
+                trans.get_command (cmd) && cmd == SCIM_TRANS_CMD_REPLY &&
+                trans.get_command (cmd) && cmd == SCIM_TRANS_CMD_OK) {
+
+                // Client is ok, return the client type.
+                return (client_type == "ConnectionTester") ? String ("") : client_type;
+            }
+        }
+    }
+    return String ("");
+}
+
 } // namespace scim
 
 /*
