@@ -3,6 +3,8 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 
 #define Uses_SCIM_EVENT
 
@@ -10,6 +12,12 @@
 #include "scim.h"
 #include "scimkeyselection.h"
 
+#ifdef GDK_WINDOWING_X11 
+#include <X11/Xlib.h>
+#include <gdk/gdkx.h>
+#include "scim_x11_utils.h"
+#endif
+
 using namespace scim;
 
 enum {
@@ -18,7 +26,6 @@ enum {
 };
 
 struct KeyGrabData {
-    std::vector <int> codes;
     KeyEvent key;
 };
 
@@ -186,7 +193,7 @@ scim_key_selection_init (ScimKeySelection *keyselection)
     gtk_tree_view_set_model (GTK_TREE_VIEW (keyselection->list_view),
                              GTK_TREE_MODEL (keyselection->list_model));
 
-    table = gtk_table_new (2, 2, FALSE);
+    table = gtk_table_new (2, 3, FALSE);
     gtk_widget_show (table);
     gtk_box_pack_start (GTK_BOX (keyselection), table, FALSE, FALSE, 0);
 
@@ -242,6 +249,24 @@ scim_key_selection_init (ScimKeySelection *keyselection)
     gtk_widget_show (keyselection->toggle_release);
     gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_release, TRUE, TRUE, 2);
 
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 2, 3,
+                      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+                      (GtkAttachOptions) (0), 4, 4);
+
+    keyselection->toggle_meta = gtk_check_button_new_with_mnemonic (_("_Meta")); 
+    gtk_widget_show (keyselection->toggle_meta);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_meta, TRUE, TRUE, 2);
+
+    keyselection->toggle_super = gtk_check_button_new_with_mnemonic (_("S_uper")); 
+    gtk_widget_show (keyselection->toggle_super);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_super, TRUE, TRUE, 2);
+
+    keyselection->toggle_hyper = gtk_check_button_new_with_mnemonic (_("_Hyper")); 
+    gtk_widget_show (keyselection->toggle_hyper);
+    gtk_box_pack_start (GTK_BOX (hbox), keyselection->toggle_hyper, TRUE, TRUE, 2);
+
     hbox = gtk_hbox_new (TRUE, 0);
     gtk_widget_show (hbox);
     gtk_box_pack_start (GTK_BOX (keyselection), hbox, FALSE, FALSE, 4);
@@ -287,6 +312,12 @@ scim_key_selection_add_key_button_callback (GtkButton        *button,
         key += String ("Alt+");
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_shift)))
         key += String ("Shift+");
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_meta)))
+        key += String ("Meta+");
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_super)))
+        key += String ("Super+");
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (keyselection->toggle_hyper)))
+        key += String ("Hyper+");
 
     key += String (gtk_entry_get_text (GTK_ENTRY (keyselection->key_code)));
 
@@ -358,10 +389,24 @@ scim_key_grab_button_callback (GtkButton        *button,
 static KeyEvent
 keyevent_gdk_to_scim (const GdkEventKey &gdkevent)
 {
-    return KeyEvent (gdkevent.keyval,
-                     (gdkevent.type == GDK_KEY_PRESS)?
-                     (gdkevent.state & ~ SCIM_KEY_ReleaseMask):
-                     (gdkevent.state | SCIM_KEY_ReleaseMask));
+    KeyEvent key;
+
+    // Use Key Symbole provided by gtk.
+    key.code = gdkevent.keyval;
+
+#ifdef GDK_WINDOWING_X11 
+    key.mask = scim_x11_keymask_x11_to_scim (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), gdkevent.state);
+#else
+    if (gdkevent.state & GDK_SHIFT_MASK) key.mask |=SCIM_KEY_ShiftMask;
+    if (gdkevent.state & GDK_LOCK_MASK) key.mask |=SCIM_KEY_CapsLockMask;
+    if (gdkevent.state & GDK_CONTROL_MASK) key.mask |=SCIM_KEY_ControlMask;
+    if (gdkevent.state & GDK_MOD1_MASK) key.mask |=SCIM_KEY_AltMask;
+    if (gdkevent.state & GDK_MOD2_MASK) key.mask |=SCIM_KEY_NumLockMask;
+#endif
+
+    if (gdkevent.type == GDK_KEY_RELEASE) key.mask |= SCIM_KEY_ReleaseMask;
+
+    return key;
 }
 
 static gboolean
@@ -369,8 +414,7 @@ scim_key_grab_press_callback (GtkDialog   *dialog,
                               GdkEventKey *event,
                               KeyGrabData *data)
 {
-    KeyEvent key = keyevent_gdk_to_scim (*event);
-    data->codes.push_back (key.code);
+    data->key = keyevent_gdk_to_scim (*event);
 
     return TRUE;
 }
@@ -382,23 +426,17 @@ scim_key_grab_release_callback (GtkDialog   *dialog,
 {
     KeyEvent key = keyevent_gdk_to_scim (*event);
 
-    data->key.mask |= (key.mask & (~ SCIM_KEY_ReleaseMask));
+    if (key.code == data->key.code) {
+        data->key.mask = key.mask;
+        if (data->key.code >= SCIM_KEY_Shift_L && data->key.code <= SCIM_KEY_Hyper_R)
+            data->key.mask |= SCIM_KEY_ReleaseMask;
+        else
+            data->key.mask &= (~ SCIM_KEY_ReleaseMask);
 
-    if (data->key.code == 0 || key.code < SCIM_KEY_Shift_L || key.code > SCIM_KEY_Hyper_R)
-        data->key.code = key.code;
-
-    for (std::vector<int>::iterator it = data->codes.begin (); it != data->codes.end (); ++it) {
-        if (*it == key.code) {
-            data->codes.erase (it);
-            break;
-        }
-    }
-
-    if (data->key.code >= SCIM_KEY_Shift_L && data->key.code <= SCIM_KEY_Hyper_R)
-        data->key.mask |= SCIM_KEY_ReleaseMask;
-
-    if (data->codes.size () == 0)
         gtk_dialog_response (dialog, GTK_RESPONSE_OK);
+    } else {
+        gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
+    }
 
     return TRUE;
 }
@@ -428,7 +466,7 @@ scim_key_selection_append_keys (ScimKeySelection *keyselection,
 
     String str;
 
-    for (int i = 0; i < keylist.size (); ++ i) {
+    for (size_t i = 0; i < keylist.size (); ++ i) {
         if (scim_key_to_string (str, keylist [i])) {
             gtk_list_store_append (keyselection->list_model, &iter);
             gtk_list_store_set (keyselection->list_model, &iter,
@@ -510,6 +548,15 @@ scim_key_selection_set_key_event (ScimKeySelection *keyselection,
     gtk_toggle_button_set_active (
         GTK_TOGGLE_BUTTON (keyselection->toggle_shift),
         event.is_shift_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_meta),
+        event.is_meta_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_super),
+        event.is_super_down ());
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (keyselection->toggle_hyper),
+        event.is_hyper_down ());
     gtk_toggle_button_set_active (
         GTK_TOGGLE_BUTTON (keyselection->toggle_release),
         event.is_key_release ());
